/*
	PROforma, PROGS Font & Raster Manager
	ProWesS, PROGS Windowing System
	DATAdesign, PROGS data handling software

	program to convert a DATAdesign file to comma separated text format
	aka the Archive export format

	This program will can only put the first value or line from the field in
	the result file. The MEMO field is not exported. Line lengths are limited
	to 80 characters. Raw fields are not output. For conformance with Archive,
	string field name are appended with a dollar sign.

	It may be necessary for this program to work to modify the fieldnames. They
	are often more limited than in DATAdesign, and often can not contain spaces.

	written by Joachim Van der Auwera
	for PROGS, Professional & Graphical Software
	started 30 January 1996
*/

#include "io_h"
#include "str_h"

#include "DATAdesign_h"
#include "ProWesS_h"

#define catch(x) if (err=(x)) return err; else

typedef struct {
	PWObject from;
	PWObject to;
} Global;

static char info[]={"This is a program to convert DATAdesign\n"
					"files to comma separated text format. Which\n"
					"is also the Archive export format.\n"
					"Please note that only the first line of each\n"
					"field can be put in the output file !"};

Error convert(PWObject object);
Error DisplayError(Error error);

Error init()
{
	Global g;
	Error err;
	PWObject outl;

	catch( PWCreate(NULL,&outl,PW_TYPE_OUTLINE,
				PW_OUTLINE_SLEEP,
				PW_OUTLINE_QUIT,
				PW_OUTLINE_QUIT_KEYPRESS, 27,
				PW_OUTLINE_ACTION_DO, convert,
				PW_WINDOW_NOSCALE,
				PW_GLOBAL_AUXILIARY, &g,
				NULL) );
	catch( PWCreate(outl,NULL,PW_TYPE_INFOSTRING,
				PW_INFOSTRING_TEXT, info,
				NULL) );

	catch( PWCreate(outl,&g.from,PW_TYPE_EDLINE,
				PW_EDLINE_MAXLENGTH, IO_MAXFULLNAME,
				NULL) );
	catch( PWCreate(outl,&g.to,PW_TYPE_EDLINE,
				PW_EDLINE_MAXLENGTH, IO_MAXFULLNAME,
				PW_EDLINE_EDLINE_PREV, g.to,
				NULL) );
	catch( PWChange(g.from,
				PW_EDLINE_EDLINE_NEXT, g.to,
				PW_EDLINE_EDLINE_AFTER, g.to,
				NULL) );
	catch( PWCreate(outl,NULL,PW_TYPE_LABEL,
				PW_LABEL_TEXT, "DATAdesign file",
				PW_POSITION_LEFT_OF, g.from,
				PW_SCALE_FACTOR, 0,
				NULL) );
	catch( PWCreate(outl,NULL,PW_TYPE_LABEL,
				PW_LABEL_TEXT, "output file",
				PW_POSITION_LEFT_OF, g.to,
				PW_SCALE_FACTOR, 0,
				NULL) );

	return PWActivate(outl);
}

static ERROR_BUILD(ErrorNoDATAdesign,"DATAdesign engine not found");
static ERROR_BUILD(ErrorDATAdesignFile,"can't open DATAdesign file");

Error convert2(Channel out)
{
	Error err;
	char types[256];

	MEMClear(256,types);

	/* start by listing the fieldnames */
	{
		short seed=0, type;
		char name[18];
		char started=FALSE;

		while (!DDcyclefield(0, &seed, &type, name))
		{
			types[seed]=type;
			if (started) catch( IOPut(out,',') ); else started=TRUE;
			catch( IOPut(out,'"') );
			catch( IOPutBytes(out,strlen(name),name,NULL) );
			if (type==2) catch( IOPut(out,'$') );
			catch( IOPut(out,'"') );
		}
		catch( IOPut(out,'\n') );
	}

	/* now convert all the records */
	{
		err=DDfirstrec(0);
		while (!err)
		{
			char started=FALSE;
			char line[80];
			int i=0;

			for (i=0 ; i<256 ; i++)
			{
				switch (types[i])
				{
				case 1: /* raw */
					if (started) catch( IOPut(out,',') ); else started=TRUE;
					catch( IOPutBytes(out,2,"\"\"",NULL) );
					break;
				case 2: /* char */
					if (started) catch( IOPut(out,',') ); else started=TRUE;
					line[0]='\0';
					DDgetline(0,i,1,line,80);
					catch( IOPut(out,'"') );
					catch( IOPutBytes(out,strlen(line),line,NULL) );
					catch( IOPut(out,'"') );
					break;
				case 3: /* short */
					if (started) catch( IOPut(out,',') ); else started=TRUE;
					{
						short value=0;
						DDgetshort(0,i,1,&value);
						STRFormatPut(line,"%d",value);
						catch( IOPutBytes(out,strlen(line),line,NULL) );
					}
					break;
				case 4: /* long */
					if (started) catch( IOPut(out,',') ); else started=TRUE;
					{
						long value=0;
						DDgetlong(0,i,1,&value);
						STRFormatPut(line,"%d",value);
						catch( IOPutBytes(out,strlen(line),line,NULL) );
					}
					break;
				case 5: /* ieee double */
					if (started) catch( IOPut(out,',') ); else started=TRUE;
					{
						double value=0;
						DDgetdouble(0,i,1,&value);
						STRFormatPut(line,"%f",value);
						catch( IOPutBytes(out,strlen(line),line,NULL) );
					}
					break;
				}
			}
			catch( IOPut(out,'\n') );
			err=DDnextrec(0);
		}
	}
	/* ..... */

	return err;
}

Error convert1(char *from, char *to)
{
	Error err;
	Channel out;

	if (DDinit()) return ERROR_CODE(ErrorNoDATAdesign);

	catch( IOOpenPath(to,OPEN_NEW,PATH_DATA,&out) );
	err=DDusefile(0,from,NULL,1);	/* view only - to caonvert ALL */
	if (err)
	{
		err=ERR_OK;
		DisplayError(ERROR_CODE(ErrorDATAdesignFile));
	}
	else
	{
		err=convert2(out);
	}

	err=DDunusebuffer(0);
	IOClose(out);
	DDfree();
	return err;
}

Error convert(PWObject object)
{
	Error err;
	Global *g;
	char from[IO_MAXFULLNAME];
	char to[IO_MAXFULLNAME];

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);

	PWChange(g->from,PW_EDLINE_GET,IO_MAXFULLNAME,from,NULL);
	PWChange(g->to,PW_EDLINE_GET,IO_MAXFULLNAME,to,NULL);

	err=convert1(from,to);
	if (err) DisplayError(err);

	return ERR_OK;
}
